﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using Ektron.Cms;
using Ektron.Cms.Common;
using System.Text.RegularExpressions;
using Microsoft.Security.Application;

public partial class Workarea_StrategyEventCancelled : Ektron.Cms.Workarea.Page
{
    protected EkMessageHelper m_refMsg;
    protected StyleHelper m_refStyle = new StyleHelper();
    protected CommonApi m_refApi = new CommonApi();

    public string StrategyName
    {
        get
        {
            if (Request.QueryString["strategy"] != null)
            {
                return AntiXss.HtmlEncode(Request.QueryString["strategy"].ToString());
            }
            return string.Empty;
        }
    }

    public string EventName
    {
        get
        {
            if (Request.QueryString["event"] != null)
            {
                return AntiXss.HtmlEncode(Request.QueryString["event"].ToString());
            }
            return string.Empty;
        }
    }

    public string CancellationMessage
    {
        get
        {
            if (Request.QueryString["msg"] != null)
            {
                return AntiXss.HtmlEncode(Request.QueryString["msg"].ToString()).Replace("&#60;br &#47;&#62;", "<br />");
            }
            return string.Empty;
        }
    }

    public string ActionName
    {
        get
        {
            if (!EventName.StartsWith("OnBefore"))
                return GetCamelSpaceString(EventName);
            return GetCamelSpaceString(EventName.Replace("OnBefore", ""));
        }
    }

    public string ObjectId
    {
        get
        {
            if (Request.QueryString["objid"] != null)
            {
                return AntiXss.HtmlEncode(Request.QueryString["objid"].ToString());
            }
            return string.Empty;
        }
    }

    /// <summary>
    /// We only want reload folder tree when content or folder deletion is cancelled by ContentAuditing or FolderAuditing strategy
    /// </summary>
    public bool ReloadFolderTree
    {
        get
        {
            return StrategyName == "Cms.Extensions.Auditing.FolderAuditing" || StrategyName == "Cms.Extensions.Auditing.ContentAuditing";
        }
    }

    protected void Page_Load(object sender, EventArgs e)
    {
        m_refMsg = m_refApi.EkMsgRef;
        //Page.Title = m_refMsg.GetMessage("generic error information msg");
        StyleSheetJS.Text = m_refStyle.GetClientScript();
        DisplayErrorToolBar();
        DisplayInfo();
    }

    private string GetCamelSpaceString(string s)
    {
        return Regex.Replace(s, "(\\B[A-Z])", " $1");
    }

    private void DisplayInfo()
    {
        lrStrategy.Text = StrategyName;
        lrMessage.Text = CancellationMessage;
        lrId.Text = ObjectId;
        lrEvent.Text = ActionName;
    }

    private void DisplayErrorToolBar()
    {
        System.Text.StringBuilder result = new System.Text.StringBuilder();
        divTitleBar.InnerHtml = "The following action was cancelled";
        result.Append("<table><tr>");
        result.Append(m_refStyle.GetButtonEventsWCaption(m_refApi.AppPath + "images/UI/Icons/back.png", "javascript:history.go(-1);", m_refMsg.GetMessage("alt back button text"), m_refMsg.GetMessage("btn back"), "onclick=\"history.go(-1);\"", StyleHelper.BackButtonCssClass, true));
        result.Append("</tr></table>");
        divToolBar.InnerHtml = result.ToString();
        result = null;
    }
}