<?xml version='1.0'?>
<xsl:stylesheet version="1.0" exclude-result-prefixes="msxsl" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:msxsl="urn:schemas-microsoft-com:xslt">

  <xsl:import href="template_buildHeader.xslt"/>

  <xsl:output method="html" version="1.0" encoding="utf-8" indent="yes" omit-xml-declaration="yes"/>

  <xsl:variable name="largeListSize" select="11"/>

  <xsl:variable name="barWidth" select="250"/>
  <!-- pixels -->
  <xsl:variable name="barColor" select="'#3163BD'"/>

  <xsl:variable name="tableHeaderClass" select="'headcell'"/>

  <xsl:variable name="percentFormat" select="'0.00%'"/>

  <xsl:variable name="fieldlist" select="/*/ektdesignpackage_list/fieldlist"/>

  <xsl:template match="Form">
    <xsl:call-template name="buildHeader"/>
    <xsl:variable name="fieldsdata" select="StatsData/*"/>
    <xsl:for-each select="$fieldlist/field[(@datalist and @datatype!='selection') or @datatype='boolean']">
      <xsl:sort select="."/>
      <xsl:variable name="field" select="."/>
      <xsl:variable name="fielddata" select="$fieldsdata[local-name()=current()/@name]"/>
      <table border="0" cellpadding="1" cellspacing="0" class="ektronGrid">
        <col align="left" />
        <col align="left" />
        <thead>
          <xsl:variable name ="dataItem" select="$fielddata/item[1]"/>
          <xsl:variable name="count" select="$dataItem/@totalCount"/>
          <tr>
            <th class="{$tableHeaderClass}" style="padding-top:5px;" align="left" colspan="2" title="{$field/@name}">
              <xsl:copy-of select="$field/node()"/>
              <xsl:value-of select="concat(' (',$count,' responses)')"/>
              <hr />
            </th>
          </tr>
        </thead>
        <tbody>
          <xsl:choose>
            <xsl:when test="@datalist">
              <!-- omit values whose count is zero if number of possible values is $largeListSize or more -->
              <xsl:for-each select="$fielddata/item">
                <xsl:if test="@hitCount &gt; 0 or last() &lt; $largeListSize">

                  <xsl:call-template name="buildResult">
                    <xsl:with-param name="value" select="@value"/>
                    <xsl:with-param name="displayValue" select="./node()"/>
                    <xsl:with-param name="count" select="@totalCount"/>
                    <xsl:with-param name="total" select="@hitCount"/>
                  </xsl:call-template>
                </xsl:if>
              </xsl:for-each>
            </xsl:when>
            <!--Checkbox-->
            <xsl:otherwise>
              <xsl:variable name="dataItem" select="$fielddata/item[@value='true']"/>
              <xsl:variable name="hitCount" select="$dataItem/@hitCount" />
              <xsl:variable name="totalCount" select="$dataItem/@totalCount" />
              <xsl:call-template name="buildResult">
                <xsl:with-param name="displayValue" select="'Checked'"/>
                <xsl:with-param name="total" select="$hitCount"/>
                <xsl:with-param name="count" select="$totalCount"/>
              </xsl:call-template>
              <xsl:call-template name="buildResult">
                <xsl:with-param name="displayValue" select="'Unchecked'"/>
                <xsl:with-param name="total" select="$totalCount - $hitCount"/>
                <xsl:with-param name="count" select="$totalCount"/>
              </xsl:call-template>
            </xsl:otherwise>
          </xsl:choose>
        </tbody>
      </table>
      <br />
    </xsl:for-each>
  </xsl:template>

  <xsl:template name="buildResult">
    <xsl:param name="value" select="''"/>
    <xsl:param name="displayValue" select="''"/>
    <xsl:param name="total" select="0"/>
    <xsl:param name="count" select="1"/>

    <xsl:variable name="percent" select="$total div $count"/>
    <tr>
      <td colspan="2" title="{$value}">
        <b>
          <xsl:copy-of select="$displayValue"/>
        </b>
      </td>
    </tr>
    <tr style="padding-bottom:5px;">
      <td title="{$total}/{$count}" style="padding-right:5px;">
        <xsl:if test="$total = 0">
            0%
        </xsl:if>
        <xsl:if test="$percent &gt; 0">
          <xsl:value-of select="format-number($percent, $percentFormat)"/>
        </xsl:if>
      </td>
      <td style="width:{$barWidth}px; height:20px">
        <xsl:if test="$total = 0">
          <div style="width:0px; height:100%; background-color:{$barColor}; border-left:solid 1px {$barColor};"
            title="0%"></div>
        </xsl:if>
        <xsl:if test="$percent &gt; 0">
          <div style="width:{round($percent * $barWidth)}px; height:100%; background-color:{$barColor}; border-left:solid 1px {$barColor};"
            title="{format-number($percent, $percentFormat)}"></div>
        </xsl:if>
      </td>
    </tr>
  </xsl:template>

</xsl:stylesheet>