
/* Start: Bug 25143 - Form report is displayed incorrectly */
 
if exists (select * FROM SYS.OBJECTS WHERE object_id = object_id(N'[dbo].[cms_getformhistogramdatafortimeperiod]') and OBJECTPROPERTY(object_id, N'IsProcedure') = 1)
drop procedure [dbo].[cms_getformhistogramdatafortimeperiod]
go
CREATE PROCEDURE [dbo].[cms_getformhistogramdatafortimeperiod] 
	-- Add the parameters for the stored procedure here
(	@formid bigint,
	@dtDateStart datetime,
	@dtDateEnd datetime,
	@fields nvarchar(500),
	@languageid int=1033,
	@totalsubmit int OUTPUT
	)
AS
BEGIN
	-- SET NOCOUNT ON added to prevent extra result sets from
	-- interfering with SELECT statements.
	SET NOCOUNT ON;

	select @totalsubmit = count(distinct form_data_id) 
	from form_data_tbl 
	where (form_id = @formid) and content_language=@languageid and date_created>@dtDateStart and date_created<@dtDateEnd

	;with tmp( form_field_name, form_field_dataitem, form_field_value) as (
	select  form_field_name, LEFT(form_field_value, CHARINDEX(',',form_field_value+',')-1),
		STUFF(form_field_value, 1, CHARINDEX(',',form_field_value+','), '')
	from form_data_tbl 
	where (form_id = @formid) and content_language=@languageid and date_created>@dtDateStart and date_created<@dtDateEnd and (form_field_name in (select * from dbo.splitComma(@fields)))
	union all
	select form_field_name, LEFT(form_field_value, CHARINDEX(',',form_field_value+',')-1),
		STUFF(form_field_value, 1, CHARINDEX(',',form_field_value+','), '')
	from tmp
	where form_field_value > ''
	)
	select form_field_name, form_field_dataitem, count(form_field_dataitem) as totalhits
	from tmp
	group by form_field_name, form_field_dataitem
	order by form_field_name, form_field_dataitem

END
go

/* End: Bug 25143 - Form report is displayed incorrectly 
========================================================*/

/* Start: Escalation 25201 - Auto alias not removed after the taxonomy is removed from the content */

if exists (select 'x' from information_schema.routines where routine_schema='dbo' and 
			routine_type = 'procedure' and routine_name = 'cms_UrlAliasReGenContentTax')
drop procedure dbo.cms_UrlAliasReGenContentTax
go

create procedure dbo.cms_UrlAliasReGenContentTax
	@p_ContentId bigint,
	@p_LanguageId int
as
set nocount on
/*
	Regenerate aliases when content moves to new taxonomy
*/
begin
	declare @v_getnextno bigint;
	declare @v_rowcount int;

	create table #taxonomies_with_content 
	(
		taxonomy_id bigint,
		taxonomy_language_id int
	);

	create table #closest_config
	(
		urlaliasconfigid bigint
	);
	
	CREATE TABLE #taxonomy_rows
	(
		rowno int identity(1,1) not null,
		rowno2 bigint null,
		urlaliasconfigid bigint NOT NULL,
		urlaliasnm nvarchar(888) collate database_default NULL,
		urltargetnm nvarchar(255) collate database_default NULL,
		Languageid int NOT NULL,
		siteid bigint NOT NULL,
		targetid bigint NOT NULL,
		nodeid bigint NOT NULL,
		aliastypeid int NOT NULL,
		IsEnabled bit null
	);

	/* Get list of taxonomies where content_id exists */
	insert into #taxonomies_with_content (taxonomy_id, taxonomy_language_id)
	select	distinct 
			t.taxonomy_id, 
			t.taxonomy_language_id
	from dbo.content co
	INNER JOIN dbo.taxonomy_item_tbl AS ti
	 ON ti.taxonomy_item_id = co.content_id 
		AND ti.taxonomy_language_id = co.content_language
	inner join dbo.taxonomy_tbl t
	 on t.taxonomy_id = ti.taxonomy_id and
		t.taxonomy_language_id = ti.taxonomy_language_id 
	where co.content_id = @p_ContentId and co.content_language = @p_LanguageId;

	/*
	Work backwards through taxonomy hierarchy getting closest UrlConfigId's that contains
	content_id
	*/
	with tax_family (taxonomy_id, 
			taxonomy_parent_id,
			taxonomy_language_id,
			taxonomy_level)
	as
	(
	select child.taxonomy_id, 
			child.taxonomy_parent_id,
			child.taxonomy_language_id,
			0 as taxonomy_level  
		from dbo.taxonomy_tbl child
		 where exists(select 'x' 
						from #taxonomies_with_content tl 
						where tl.taxonomy_id = child.taxonomy_id and 
						tl.taxonomy_language_id = child.taxonomy_language_id)
	union all
	select ancestors.taxonomy_id, 
			ancestors.taxonomy_parent_id,
			ancestors.taxonomy_language_id,
			tf.taxonomy_level+1 
		from dbo.taxonomy_tbl ancestors
		inner join tax_family as tf
		 on (tf.taxonomy_parent_id = ancestors.taxonomy_id and 
			tf.taxonomy_language_id = ancestors.taxonomy_language_id and ancestors.taxonomy_id <> 0)
	)
	insert into #closest_config (UrlAliasConfigId)
	select ordered.UrlAliasConfigId 
	from (
		select cfg.urlaliasconfigid, tf.taxonomy_language_id,
		ROW_NUMBER() over (partition by cfg.urlaliasconfigid order by tf.taxonomy_level) ConfigOrder from tax_family tf
		inner join dbo.UrlAliasConfig cfg
		on cfg.SourceId = tf.taxonomy_id and 
			cfg.LanguageId = tf.taxonomy_language_id and 
			cfg.UrlAliasConfigTypeId = 1
		) ordered
	where ordered.ConfigOrder = 1;
	
	/*
		Generate taxonomy rows for content_id within the closest urlaliasconfigs
	*/
	insert into #taxonomy_rows (UrlAliasConfigId, urlaliasnm, urltargetnm, Languageid, SiteID, TargetID, NodeID, AliasTypeID, IsEnabled)
	select 	b.urlaliasconfigid,
		CASE	WHEN (b.PageTypeId = 1) THEN		
							dbo.cms_UrlAliasBldTaxonomy(a.taxonomy_path, d.content_title, fe.FileExtension,b.ExcludedPathId,b.LanguageId,b.urlaliasconfigid) 
				WHEN (b.PageTypeId = 2) THEN		
							dbo.cms_UrlAliasBldTaxonomy(a.taxonomy_path, CAST(d.content_id as varchar), fe.FileExtension,b.ExcludedPathId,b.LanguageId,b.urlaliasconfigid)
				WHEN (b.PageTypeId = 3)THEN		
							dbo.cms_UrlAliasBldTaxonomy(a.taxonomy_path, CAST(d.content_id as varchar) + '/' + CAST(d.content_language as varchar), fe.FileExtension,b.ExcludedPathId,b.LanguageId,b.urlaliasconfigid)
				ELSE
							dbo.cms_UrlAliasBldTaxonomy(a.taxonomy_path, d.content_title, fe.FileExtension,b.ExcludedPathId,b.LanguageId,b.urlaliasconfigid) 
		END as urlaliasnm,
		
		CASE 
			when tmpl.template_filename is not null and b.UseTemplate = 2 and ISNULL(b.SourceParmName,'') = '' AND CHARINDEX('?',tmpl.template_filename) > 0
				THEN stuff(f.filename,1,charindex(N'?',f.filename), tmpl.template_filename + N'&')
			WHEN tmpl.template_filename is not null and b.UseTemplate = 2 and ISNULL(b.SourceParmName,'') = '' AND CHARINDEX('?',tmpl.template_filename) = 0
				THEN stuff(f.filename,1,charindex(N'?',f.filename)-1,tmpl.template_filename)
			when tmpl.template_filename is not null and b.UseTemplate = 2 and ISNULL(b.SourceParmName,'') != '' AND CHARINDEX('?',tmpl.template_filename) > 0
				then dbo.BuildTaxonomyTargetLink(b.SourceParmName,a.taxonomy_id,stuff(f.filename,1,charindex(N'?',f.filename), tmpl.template_filename + N'&'))
			when tmpl.template_filename is not null and b.UseTemplate = 2 and ISNULL(b.SourceParmName,'') != '' AND CHARINDEX('?',tmpl.template_filename) = 0
				then dbo.BuildTaxonomyTargetLink(b.SourceParmName,a.taxonomy_id,(stuff(f.filename,1,charindex(N'?',f.filename)-1,tmpl.template_filename)))
			when b.UseTemplate = 1 and ISNULL(b.SourceParmName,'') != ''
				then dbo.BuildTaxonomyTargetLink(b.SourceParmName,a.taxonomy_id,f.filename)
			ELSE
				f.filename
		END
		as urltargetnm,
		
		d.content_language as Languageid,
		b.siteid, 
		   d.content_id as targetid,
		   a.taxonomy_id as nodeid,
		   b.UrlAliasConfigTypeId as aliastypeid,
		   IsEnabled
	FROM dbo.UrlAliasConfig b
	INNER JOIN #closest_config parents
		on parents.urlaliasconfigid = b.urlaliasconfigid
	INNER JOIN dbo.taxonomy_children_tbl tct 
				ON b.SourceId = tct.taxonomy_id
			   AND b.UrlAliasConfigTypeId = 1 
			   AND tct.taxonomy_language_id = b.LanguageId
	INNER JOIN dbo.taxonomy_tbl AS a 
	 on a.taxonomy_id = tct.taxonomy_child_id and 
		tct.taxonomy_language_id = a.taxonomy_language_id 
			INNER JOIN dbo.taxonomy_item_tbl AS c 
					ON a.taxonomy_id = c.taxonomy_id 
				   AND a.taxonomy_language_id = c.taxonomy_language_id 
			INNER JOIN dbo.content AS d 
					ON c.taxonomy_item_id = d.content_id 
				   AND c.taxonomy_language_id = d.content_language
				   and d.content_language = @p_LanguageId
				   and d.content_id = @p_ContentId 
			INNER JOIN dbo.content_folder_tbl AS e 
					ON d.folder_id = e.folder_id 
				   AND e.folder_type NOT IN (3,4)
			INNER JOIN dbo.library AS f 
					ON d.content_id = f.content_id 
				   AND d.content_language = f.content_language 
				   AND f.libtype_id NOT IN (1,3) 		
				   AND f.content_type NOT IN (7)
			inner join dbo.UrlAliasFileExtensions fe
			 on fe.FileExtensionId = b.FileExtensionId
			left outer join dbo.templates_tbl tmpl
			 on tmpl.template_id = a.template_id and 
				tmpl.content_language = a.taxonomy_language_id
			where b.IsDeleted = 0
	option (force order)
	;
	
	select @v_rowcount = count(*) from #taxonomy_rows;
	
	/* Delete pre-existing rows prior to reinsert */
	Delete from dbo.urlaliasmapping where targetid = @p_ContentId and aliastypeid = 1 and Languageid = @p_LanguageId;
		
	/*
		If there is nothing to map at this point get out and report. Better than changing MAX_ENTRIES pointer and failing there.
	*/
	if (@v_rowcount = 0) RETURN;
		
	exec cms_getnexttableid @tablename = 'urlaliasmapping', @batchsize = @v_rowcount, @newvalue = @v_getnextno output 
	
	-- offset for row_number() starting at 1
	set @v_getnextno = @v_getnextno - 1;
	
	update #taxonomy_rows set rowno2 = rowno+@v_getnextno;
	
	with CheckForDuplicateAliasName (UrlAliasMappingId, urlaliasnm)
	as
	(
		select rowno2, urlaliasnm
			from #taxonomy_rows t
		where exists (select 'x' from UrlAliasMapping map where map.urlaliasnm = t.urlaliasnm and
						map.Languageid = t.Languageid and
						map.SiteID = t.SiteID)
	)
	update tax
		set urlaliasnm = dbo.cms_UrlAliasResolveTitle(tax.urlaliasnm,tax.UrlAliasConfigId,tax.rowno2)
	from #taxonomy_rows tax
	inner join CheckForDuplicateAliasName CFD
	 on CFD.UrlAliasMappingId = tax.rowno2
	  ;
	
	insert into dbo.UrlAliasMapping (UrlAliasMappingId, UrlAliasConfigId, urlaliasnm, urltargetnm, Languageid, SiteID, TargetID, NodeID, AliasTypeID, IsEnabled)
	select 
		rowno2 as UrlAliasMappingId,
		urlaliasconfigid, 
		urlaliasnm, 
		urltargetnm, 
		languageid, 
		siteid, 
		targetid, 
		nodeid, 
		aliastypeid,
		IsEnabled
	from #taxonomy_rows;
	
	drop table #taxonomies_with_content;
	drop table #closest_config;
	drop table #taxonomy_rows;

end
go

/* End: Escalation 25201 - Auto alias not removed after the taxonomy is removed from the content 
================================================================================================*/

/* Start: Escalation 25229:Duplicate Rows created in scope_info_history_locker and scope_loginfo_locker on each sync
================================================================================================*/

if exists (select 'x' from information_schema.routines where routine_name = 'sync_LoadParallelEntityScopeModel')
drop procedure dbo.sync_LoadParallelEntityScopeModel
go

create procedure [dbo].sync_LoadParallelEntityScopeModel 
(
	@p_TargetModel int, 
	@p_EntityIdHashKey nvarchar(32)
)
as
set nocount on 
begin	
	/*
	The proc is called on both the sending and receiving database before ANY esync activity happens since v8.6.1.
	It backs up the rows in the base scope tables to the locker tables and if needed will load the right scope model
	rows from the locker tables to the base scope tables.

	Anything other than zero as a return value indicates a problem:

		Values returned between -10 and -1 are application defined errors capturing problems that are forseeable.
			Example: -1 is the error returned if the value supplied to @p_TargetModel is invalid.
		 
		Values less than -10 are actually positive database error codes and should be debugged as such.
			Example: -2627 is actually DB ERROR 2627 (Primary key violation). 
	*/
	begin transaction;
	begin try

	declare @result int;
	declare @steperror int;
	declare @v_ErrorLog table
		(	scope_chglog_id uniqueidentifier default(newid()) not null,
			scope_chglog_procnm nvarchar(128) null,
			scope_chglog_target_from int null,
			scope_chglog_target_from_hashcd nvarchar(38) null,
			scope_chglog_target_to int null,
			scope_chglog_target_to_hashcd nvarchar(32) null,
			scope_chglog_step_cd int not null,
			scope_chglog_step_nm nvarchar(128) not null,
			scope_chglog_proc_cd int null,
			scope_chglog_error_cd int null,
			scope_chglog_error_msg nvarchar(2048) not null,
			scope_chglog_dtm datetime default GETUTCDATE() not null,
			scope_chglog_row_cnt int null
		);
	declare @v_procname nvarchar(128);
	declare @v_stepname nvarchar(128);
	declare @v_errorcd int;
	declare @v_errormsg nvarchar(2048);
	declare @v_EntityIdHashKey nvarchar(32);
	declare @v_rows int;
	declare @v_CurrentlyLoadedScopeRows int;
	declare @v_DetailMessage nvarchar(2048);
	declare @v_ScopesWithinModel int;
	
	set @v_procname = OBJECT_NAME(@@PROCID);
	set @v_EntityIdHashKey = @p_EntityIdHashKey;
			

	set @v_ScopesWithinModel = 1
	if (@p_TargetModel = 1)
	BEGIN
		set @v_ScopesWithinModel = 6 --the scopes in master: aspnet, ektron, history, notification, search, workflow, and optionally custom
	END

	
	set @result = 0;
		
		/*  1. detect if model exists in scope_locker_manager
			2. find record in scope_locker_history indicating which scope is currently active
			3. take current data and save it back to the locker for the correct type 
			4. delete current data from tables
			5. restore type of data selected from lockers into the tables 
			6. insert new row into scope_locker_history 
	
		Step 1: Check that the target exists

		Check that the target exists, if not log it and exit with a result of -1 */
	


	set @v_stepname='Step1: Check target exists';

	if not exists(select 'X' from scope_locker_manager where id = @p_TargetModel) 
	begin
		set @result = -1;
			
		insert into @v_ErrorLog (scope_chglog_procnm, scope_chglog_target_from, scope_chglog_target_to, scope_chglog_step_cd, 
								 scope_chglog_step_nm, scope_chglog_proc_cd, scope_chglog_error_cd, scope_chglog_error_msg,
								 scope_chglog_target_from_hashcd, scope_chglog_target_to_hashcd)
		values (@v_procname, 0, @p_TargetModel, @result, @v_stepname, @result, @result, 'skipped - Target Model does not exist' ,NULL,@v_EntityIdHashKey);
	end;
	
	if(@result < 0)
	begin
		IF @@TRANCOUNT > 0
		    rollback transaction;
			
		insert into dbo.scope_chglog_tbl 
		  	   (scope_chglog_id, scope_chglog_procnm, scope_chglog_target_from, scope_chglog_target_to, 
			    scope_chglog_step_cd, scope_chglog_step_nm, scope_chglog_proc_cd, scope_chglog_error_cd, 
			    scope_chglog_error_msg, scope_chglog_target_from_hashcd, scope_chglog_target_to_hashcd,
			    scope_chglog_dtm, scope_chglog_row_cnt)
		select scope_chglog_id, scope_chglog_procnm, scope_chglog_target_from, scope_chglog_target_to, 
		        scope_chglog_step_cd, scope_chglog_step_nm, scope_chglog_proc_cd, scope_chglog_error_cd, 
		        scope_chglog_error_msg, scope_chglog_target_from_hashcd, scope_chglog_target_to_hashcd,
		        scope_chglog_dtm, scope_chglog_row_cnt
		from @v_ErrorLog
		order by scope_chglog_dtm asc;
	
		return @result;
	end
	
			
	insert into @v_ErrorLog (scope_chglog_procnm, scope_chglog_target_from, scope_chglog_target_to, scope_chglog_step_cd, 
							 scope_chglog_step_nm, scope_chglog_proc_cd, scope_chglog_error_cd, scope_chglog_error_msg,
							 scope_chglog_target_from_hashcd, scope_chglog_target_to_hashcd)
	values (@v_procname, 0, @p_TargetModel, @result, @v_stepname, @result, @result, 'success' ,NULL, @v_EntityIdHashKey);

	--scope_info table should not have any default rows. These rows are only created when sync runs.
	delete from scope_info_locker where isdefault = 1;

	/* !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
		Step2: Backup scopes from base tables to lockers
		       Whether the currentl scope model is the same as the target model OR NOT we still backup the rows in either case.
			   No need to duplicate the code for this.
       !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	*/
	set @steperror = -2;
	set	@v_stepname='Step2: Backup scopes';

	--variable name changed. a scope model contains scopes. it is not the same as a scope.
	declare @v_currentlyActiveScopeModel int;
	declare @v_currentlyActiveScopeHash nvarchar(32);
	
	select top 1 @v_currentlyActiveScopeModel = scope_id, @v_currentlyActiveScopeHash = scope_filter_hashcd from scope_locker_history where scope_id > -1 order by date_loaded desc;
	set @v_CurrentlyLoadedScopeRows = @@rowcount;
	
	if (@v_currentlyActiveScopeModel is not null) --if anything is currently loaded.
	BEGIN
		--scope_config
		insert into scope_config_locker (
			scope_manager_id, 
			scope_filter_hashcd, 
			scope_name, 
			scope_table, 
			scope_description, 
			scope_sync_on, 
			scope_after_event)
		select 
			@v_currentlyActiveScopeModel, 
			@v_currentlyActiveScopeHash, 
			scope_config.scope_name, 
			scope_config.scope_table, 
			scope_config.scope_description, 
			1, 
			scope_config.scope_after_event 
		from scope_config
		where not exists(
			select 'X' 
			from scope_config_locker 
			where scope_config_locker.scope_manager_id = @v_currentlyActiveScopeModel 
			and ((scope_config_locker.scope_filter_hashcd = @v_currentlyActiveScopeHash) or (scope_config_locker.scope_filter_hashcd is null and @v_currentlyActiveScopeHash is null))
			and scope_config_locker.scope_name = scope_config.scope_name  
			and scope_config_locker.IsDefault = 0)

		update scope_config_locker
		set scope_config_locker.scope_table = scope_config.scope_table,
			scope_config_locker.scope_description = scope_config.scope_description,
			scope_config_locker.scope_sync_on = scope_config.scope_sync_on,
			scope_config_locker.scope_after_event = scope_config.scope_after_event
		from scope_config
		where scope_config_locker.scope_manager_id = @v_currentlyActiveScopeModel 
			and ((scope_config_locker.scope_filter_hashcd = @v_currentlyActiveScopeHash) or (scope_config_locker.scope_filter_hashcd is null and @v_currentlyActiveScopeHash is null))
			and scope_config_locker.scope_name = scope_config.scope_name  
			and scope_config_locker.IsDefault = 0

		--scope_info
		insert into scope_info_locker (
			scope_manager_id, 
			scope_filter_hashcd, 
			sync_scope_local_id, 
			sync_scope_id, 
			sync_scope_name, 
			sync_scope_knowledge, 
			sync_scope_cleanup_knowledge, 
			sync_scope_datetime, 
			sync_scope_timestamp, 
			scope_cleanup_timestamp, 
			scope_restore_count)
		select 
			@v_currentlyActiveScopeModel, 
			@v_currentlyActiveScopeHash, 
			scope_info.sync_scope_local_id, 
			scope_info.sync_scope_id, 
			scope_info.sync_scope_name, 
			scope_info.sync_scope_knowledge, 
			scope_info.sync_scope_cleanup_knowledge, 
			scope_info.sync_scope_datetime, 
			scope_info.sync_scope_timestamp, 
			scope_info.scope_cleanup_timestamp, 
			scope_info.scope_restore_count 
		from scope_info
		where not exists(
			select 'X' 
			from scope_info_locker 
			where scope_info_locker.scope_manager_id = @v_currentlyActiveScopeModel 
			and ((scope_info_locker.scope_filter_hashcd = @v_currentlyActiveScopeHash) or (scope_info_locker.scope_filter_hashcd is null and @v_currentlyActiveScopeHash is null))
			and scope_info_locker.sync_scope_local_id = scope_info.sync_scope_local_id  
			and scope_info_locker.sync_scope_id = scope_info.sync_scope_id  
			and scope_info_locker.sync_scope_name = scope_info.sync_scope_name  
			and scope_info_locker.IsDefault = 0)

		update scope_info_locker
		set 
			scope_info_locker.sync_scope_knowledge = scope_info.sync_scope_knowledge,
			scope_info_locker.sync_scope_cleanup_knowledge = scope_info.sync_scope_cleanup_knowledge, 
			scope_info_locker.sync_scope_datetime = scope_info.sync_scope_datetime, 
			scope_info_locker.sync_scope_timestamp = scope_info.sync_scope_timestamp, 
			scope_info_locker.scope_cleanup_timestamp = scope_info.scope_cleanup_timestamp, 
			scope_info_locker.scope_restore_count = scope_info.scope_cleanup_timestamp
		from scope_info
		where scope_info_locker.scope_manager_id = @v_currentlyActiveScopeModel 
			and ((scope_info_locker.scope_filter_hashcd = @v_currentlyActiveScopeHash) or (scope_info_locker.scope_filter_hashcd is null and @v_currentlyActiveScopeHash is null))
			and scope_info_locker.sync_scope_local_id = scope_info.sync_scope_local_id  
			and scope_info_locker.sync_scope_id = scope_info.sync_scope_id  
			and scope_info_locker.sync_scope_name = scope_info.sync_scope_name  
			and scope_info_locker.IsDefault = 0

		--backup scope_info_history
		insert into scope_info_history_locker (
			scope_manager_id, 
			scope_filter_hashcd, 
			sync_scope_historyid, 
			sync_scope_id, 
			sync_scope_name, 
			sync_scope_knowledge, 
			sync_scope_cleanup_knowledge, 
			sync_scope_schema, 
			sync_scope_history_datetime)
		select 
			@v_currentlyActiveScopeModel, 
			@v_currentlyActiveScopeHash, 
			scope_info_history.sync_scope_historyid, 
			scope_info_history.sync_scope_id, 
			scope_info_history.sync_scope_name, 
			scope_info_history.sync_scope_knowledge, 
			scope_info_history.sync_scope_cleanup_knowledge, 
			scope_info_history.sync_scope_schema, 
			scope_info_history.sync_scope_history_datetime 
		from scope_info_history
		where not exists (select 'X' from scope_info_history_locker l 
						  where @v_currentlyActiveScopeModel = l.scope_manager_id and 
								((@v_currentlyActiveScopeHash = l.scope_filter_hashcd) or (l.scope_filter_hashcd is null and @v_currentlyActiveScopeHash is null)) and
								l.sync_scope_id = scope_info_history.sync_scope_id and 
								l.sync_scope_history_datetime = scope_info_history.sync_scope_history_datetime);

		--backup scope_loginfo
		insert into scope_loginfo_locker (
			scope_manager_id, 
			scope_filter_hashcd, 
			sync_log_id, 
			sync_scope_id, 
			sync_start_version, 
			sync_end_version, 
			sync_provider_type, 
			sync_changes_total, 
			sync_changes_applied, 
			sync_changes_failed, 
			sync_user_id, 
			date_created, 
			sync_start_time, 
			sync_end_time, 
			sync_scope_position, 
			sync_scope_conflict, 
			sync_profile_id, 
			sync_scope_direction, 
			sync_conflict_policy)
		select 
			@v_currentlyActiveScopeModel, 
			@v_currentlyActiveScopeHash, 
			scope_loginfo.sync_log_id, 
			scope_loginfo.sync_scope_id, 
			scope_loginfo.sync_start_version, 
			scope_loginfo.sync_end_version, 
			scope_loginfo.sync_provider_type, 
			scope_loginfo.sync_changes_total, 
			scope_loginfo.sync_changes_applied, 
			scope_loginfo.sync_changes_failed, 
			scope_loginfo.sync_user_id, 
			scope_loginfo.date_created, 
			scope_loginfo.sync_start_time, 
			scope_loginfo.sync_end_time, 
			scope_loginfo.sync_scope_position, 
			scope_loginfo.sync_scope_conflict, 
			scope_loginfo.sync_profile_id, 
			scope_loginfo.sync_scope_direction, 
			scope_loginfo.sync_conflict_policy 
		from scope_loginfo
		where not exists (select 'X' from scope_loginfo_locker l 
						  where @v_currentlyActiveScopeModel = l.scope_manager_id and
								((@v_currentlyActiveScopeHash = l.scope_filter_hashcd) or (l.scope_filter_hashcd is null and @v_currentlyActiveScopeHash is null)) and
								l.sync_scope_id = scope_loginfo.sync_scope_id and 
								l.sync_start_time = scope_loginfo.sync_start_time);

		--backup scope_table_map - static except for custom scope only.
		if (@v_currentlyActiveScopeModel = 1 and (exists (select 'x' from scope_table_map where scope_name = 'custom')))
		BEGIN
			delete from scope_table_map_locker where scope_name = 'custom';
		
			insert into scope_table_map_locker (
				scope_manager_id, 
				scope_name, 
				table_name, 
				table_order)
			select 
				1, 
				'custom', 
				scope_table_map.table_name, 
				scope_table_map.table_order 
			from scope_table_map 
			where scope_name = 'custom';
		END
	END
		
	insert into @v_ErrorLog (scope_chglog_procnm, scope_chglog_target_from, scope_chglog_target_to, scope_chglog_step_cd, 
							 scope_chglog_step_nm, scope_chglog_proc_cd, scope_chglog_error_cd, scope_chglog_error_msg,
							 scope_chglog_target_from_hashcd, scope_chglog_target_to_hashcd)
	values (@v_procname, @v_currentlyActiveScopeModel, @p_TargetModel, @result, @v_stepname, @result, @result, 'success' ,@v_currentlyActiveScopeHash, @v_EntityIdHashKey);

	
	/* !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
		Step 3: Get current scope model

		Get current scope model that is in use. If it is the same as the target exit as no change is required.
	  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	*/
	set @steperror = -3;
	set	@v_stepname='Step3: Get current scope model';		
	
	if (@v_currentlyActiveScopeModel = @p_TargetModel and coalesce(@v_currentlyActiveScopeHash,'0') = coalesce(@v_EntityIdHashKey,'0') and @v_CurrentlyLoadedScopeRows <> 0) 
	BEGIN
		if(
			@p_TargetModel <> 1 --we're ok if we're not going to 1
			or
			exists(select 'x' from scope_info where sync_scope_name = 'ektron') --if we are goint to 1 then we should have an ektron scope
			or
			not exists(select 'x' from scope_loginfo_locker where scope_manager_id = 1 and scope_filter_hashcd is null and IsDefault = 0) --if we don't have an ektron scope, we should never have run it before
		)
		begin
			set @result = 0;

			insert into @v_ErrorLog (scope_chglog_procnm, scope_chglog_target_from, scope_chglog_target_to, scope_chglog_step_cd, 
									 scope_chglog_step_nm, scope_chglog_proc_cd, scope_chglog_error_cd, scope_chglog_error_msg,
									 scope_chglog_target_from_hashcd, scope_chglog_target_to_hashcd)
			values (@v_procname, @v_currentlyActiveScopeModel, @p_TargetModel, @result, @v_stepname, @result, @result, 'success - Scope Model change skipped as it has not changed.' ,@v_currentlyActiveScopeHash, @v_EntityIdHashKey);
			
			IF @@TRANCOUNT > 0
				COMMIT TRANSACTION;
			
			insert into dbo.scope_chglog_tbl 
				   (scope_chglog_id, scope_chglog_procnm, scope_chglog_target_from, scope_chglog_target_to, 
					scope_chglog_step_cd, scope_chglog_step_nm, scope_chglog_proc_cd, scope_chglog_error_cd, 
					scope_chglog_error_msg, scope_chglog_target_from_hashcd, scope_chglog_target_to_hashcd,
					scope_chglog_dtm, scope_chglog_row_cnt)
			select scope_chglog_id, scope_chglog_procnm, scope_chglog_target_from, scope_chglog_target_to, 
					scope_chglog_step_cd, scope_chglog_step_nm, scope_chglog_proc_cd, scope_chglog_error_cd, 
					scope_chglog_error_msg, scope_chglog_target_from_hashcd, scope_chglog_target_to_hashcd,
					scope_chglog_dtm, scope_chglog_row_cnt
			from @v_ErrorLog
			order by scope_chglog_dtm asc;
		
			return @result;
		end

		--if we're here then it means we don't have the ektron row in scope_info. we'll restore from the locker, then check if we have it - if we don't we'll bomb
	END
		
	insert into @v_ErrorLog (scope_chglog_procnm, scope_chglog_target_from, scope_chglog_target_to, scope_chglog_step_cd, 
							 scope_chglog_step_nm, scope_chglog_proc_cd, scope_chglog_error_cd, scope_chglog_error_msg,
							 scope_chglog_target_from_hashcd, scope_chglog_target_to_hashcd)
	values (@v_procname, @v_currentlyActiveScopeModel, @p_TargetModel, @result, @v_stepname, @result, @result, 'success' ,@v_currentlyActiveScopeHash, @v_EntityIdHashKey);
	
	/**************************************
		Step4: Remove current scope
	***************************************/
	set @steperror = -4;
	set	@v_stepname='Step4: Remove current scope';
	
	delete from scope_config;
	delete from scope_info;
	delete from scope_info_history;
	delete from scope_loginfo;
	delete from scope_table_map;
		
	insert into @v_ErrorLog (scope_chglog_procnm, scope_chglog_target_from, scope_chglog_target_to, scope_chglog_step_cd, 
							 scope_chglog_step_nm, scope_chglog_proc_cd, scope_chglog_error_cd, scope_chglog_error_msg,
							 scope_chglog_target_from_hashcd, scope_chglog_target_to_hashcd)
	values (@v_procname, @v_currentlyActiveScopeModel, @p_TargetModel, @result, @v_stepname, @result, @result, 'success' ,@v_currentlyActiveScopeHash, @v_EntityIdHashKey);
	
	/**************************************
		Step5: Restore requested target scope
	***************************************/
	set @steperror = -5;
	set	@v_stepname='Step5: Restore requested target scope';
	
	/**************************************
		S5: SCOPE_CONFIG RESTORE - static table between scope models.
	***************************************/

	set @v_rows = 0;
	
	;WITH results AS
	(SELECT *, ROW_NUMBER() 
		OVER (PARTITION BY scope_name ORDER BY isdefault ASC, scope_filter_hashcd desc) AS rn
		FROM scope_config_locker
		WHERE scope_manager_id = @p_TargetModel and (scope_filter_hashcd = @v_EntityIdHashKey or scope_filter_hashcd is null)
	)
	insert into dbo.scope_config ( scope_name, scope_table, scope_description, scope_sync_on, scope_after_event )
	select results.scope_name, results.scope_table, results.scope_description, results.scope_sync_on, results.scope_after_event
	from results
	WHERE results.rn = 1;

	set @v_rows = @@rowcount;

	set @v_DetailMessage = '   '+@v_stepname + ' scope_config.'; 
		
	insert into @v_ErrorLog (scope_chglog_procnm, scope_chglog_target_from, scope_chglog_target_to, scope_chglog_step_cd, 
							 scope_chglog_step_nm, scope_chglog_proc_cd, scope_chglog_error_cd, scope_chglog_error_msg,
							 scope_chglog_target_from_hashcd, scope_chglog_target_to_hashcd, scope_chglog_row_cnt)
	values (@v_procname, @v_currentlyActiveScopeModel, @p_TargetModel, @result, @v_stepname, @result, @result, @v_DetailMessage ,@v_currentlyActiveScopeHash, @v_EntityIdHashKey, @v_rows);
	

	if((select COUNT(*) from scope_config where scope_name not like 'custom') < @v_ScopesWithinModel)
	BEGIN
	/* we are only here because we are missing rows. this is a problem though, we never should be.	*/		
		set @result = @steperror;

		insert into @v_ErrorLog (scope_chglog_procnm, scope_chglog_target_from, scope_chglog_target_to, scope_chglog_step_cd, 
									scope_chglog_step_nm, scope_chglog_proc_cd, scope_chglog_error_cd, scope_chglog_error_msg,
									scope_chglog_target_from_hashcd, scope_chglog_target_to_hashcd, scope_chglog_row_cnt)
		values (@v_procname, @v_currentlyActiveScopeModel, @p_TargetModel, @result, @v_stepname, @result, @result, @v_DetailMessage ,@v_currentlyActiveScopeHash, @v_EntityIdHashKey, @v_rows);
		
		IF @@TRANCOUNT > 0
		ROLLBACK transaction;
	
		insert into dbo.scope_chglog_tbl 
				(scope_chglog_id, scope_chglog_procnm, scope_chglog_target_from, scope_chglog_target_to, 
				scope_chglog_step_cd, scope_chglog_step_nm, scope_chglog_proc_cd, scope_chglog_error_cd, 
				scope_chglog_error_msg, scope_chglog_target_from_hashcd, scope_chglog_target_to_hashcd,
				scope_chglog_dtm, scope_chglog_row_cnt)
		select scope_chglog_id, scope_chglog_procnm, scope_chglog_target_from, scope_chglog_target_to, 
				scope_chglog_step_cd, scope_chglog_step_nm, scope_chglog_proc_cd, scope_chglog_error_cd, 
				scope_chglog_error_msg, scope_chglog_target_from_hashcd, scope_chglog_target_to_hashcd,
				scope_chglog_dtm, scope_chglog_row_cnt
		from @v_ErrorLog
		order by scope_chglog_dtm asc;

		return @result;
	END;

	
	
	/**************************************		
		S5: SCOPE_INFO RESTORE
		
			1. Try with Hashcd (filters) - non default rows - only for non master scope model
			2. Try without hashcd - non default rows
			3. Try default 	
	***************************************/
	
	disable trigger dbo.scope_info_sync_scope_timestamp_trigger ON dbo.scope_info;
	
	;WITH results AS
	(SELECT *, ROW_NUMBER() 
		OVER (PARTITION BY sync_scope_name ORDER BY isdefault ASC, scope_filter_hashcd desc, sync_scope_datetime desc) AS rn
		FROM scope_info_locker
		WHERE scope_manager_id = @p_TargetModel and (scope_filter_hashcd = @v_EntityIdHashKey or scope_filter_hashcd is null)
	)
	insert into scope_info ( 
		sync_scope_local_id, 
		sync_scope_id, 
		sync_scope_name, 
		sync_scope_knowledge, 
		sync_scope_cleanup_knowledge, 
		sync_scope_datetime, 
		sync_scope_timestamp, 
		scope_restore_count )
	select 
		results.sync_scope_local_id, 
		results.sync_scope_id, 
		results.sync_scope_name, 
		results.sync_scope_knowledge, 
		results.sync_scope_cleanup_knowledge, 
		results.sync_scope_datetime, 
		results.sync_scope_timestamp, 
		results.scope_restore_count
	from results
	WHERE results.rn = 1;

	set @v_rows = @@rowcount;
	set @v_DetailMessage = '   '+@v_stepname + ' scope_info.'; 
		
	insert into @v_ErrorLog (scope_chglog_procnm, scope_chglog_target_from, scope_chglog_target_to, scope_chglog_step_cd, 
							 scope_chglog_step_nm, scope_chglog_proc_cd, scope_chglog_error_cd, scope_chglog_error_msg,
							 scope_chglog_target_from_hashcd, scope_chglog_target_to_hashcd, scope_chglog_row_cnt)
	values (@v_procname, @v_currentlyActiveScopeModel, @p_TargetModel, @result, @v_stepname, @result, @result, @v_DetailMessage ,@v_currentlyActiveScopeHash, @v_EntityIdHashKey, @v_rows);
	
	enable trigger dbo.scope_info_sync_scope_timestamp_trigger ON dbo.scope_info;

	--do a check to ensure that if we are doing target model 1 that we have an ektron row. if we've run a sync before, then we should have one.
	if(
		@p_TargetModel = 1
		and
		not exists(select 'x' from scope_info where sync_scope_name = 'ektron')
		and
		not exists(select 'x' from scope_info_locker where sync_scope_name = 'ektron' and IsDefault = 0 and scope_filter_hashcd is null and scope_manager_id = 1)
		and
		exists(select 'x' from scope_loginfo_locker where scope_manager_id = 1 and scope_filter_hashcd is null and IsDefault = 0)
	)
	begin
		--in this case we we are screwed
		set @result = -1;

		insert into @v_ErrorLog (scope_chglog_procnm, scope_chglog_target_from, scope_chglog_target_to, scope_chglog_step_cd, 
									scope_chglog_step_nm, scope_chglog_proc_cd, scope_chglog_error_cd, scope_chglog_error_msg,
									scope_chglog_target_from_hashcd, scope_chglog_target_to_hashcd)
		values (@v_procname, 0, @p_TargetModel, @result, @v_stepname, @result, @result, 'Error - Ektron scope_info row does not appear to exist' ,NULL,@v_EntityIdHashKey);
		IF @@TRANCOUNT > 0
			rollback transaction;
			
		insert into dbo.scope_chglog_tbl 
		  		(scope_chglog_id, scope_chglog_procnm, scope_chglog_target_from, scope_chglog_target_to, 
				scope_chglog_step_cd, scope_chglog_step_nm, scope_chglog_proc_cd, scope_chglog_error_cd, 
				scope_chglog_error_msg, scope_chglog_target_from_hashcd, scope_chglog_target_to_hashcd,
				scope_chglog_dtm, scope_chglog_row_cnt)
		select scope_chglog_id, scope_chglog_procnm, scope_chglog_target_from, scope_chglog_target_to, 
				scope_chglog_step_cd, scope_chglog_step_nm, scope_chglog_proc_cd, scope_chglog_error_cd, 
				scope_chglog_error_msg, scope_chglog_target_from_hashcd, scope_chglog_target_to_hashcd,
				scope_chglog_dtm, scope_chglog_row_cnt
		from @v_ErrorLog
		order by scope_chglog_dtm asc;
	
		return @result;
	end


	/**************************************
		S5: SCOPE_INFO_HISTORY RESTORE - skipping this as it is not required.
		S5: SCOPE_LOGINFO RESTORE - skipping this as it is not required.
	***************************************/
	
	/**************************************		
		S5: SCOPE_TABLE_MAP RESTORE
		
		Hash codes do not apply to this table	
	***************************************/

	insert into scope_table_map ( scope_name, table_name, table_order )
	select scl.scope_name, scl.table_name, scl.table_order
	from scope_table_map_locker scl
	where scl.scope_manager_id = @p_TargetModel;
	
	insert into @v_ErrorLog (scope_chglog_procnm, scope_chglog_target_from, scope_chglog_target_to, scope_chglog_step_cd, 
							 scope_chglog_step_nm, scope_chglog_proc_cd, scope_chglog_error_cd, scope_chglog_error_msg,
							 scope_chglog_target_from_hashcd, scope_chglog_target_to_hashcd)
	values (@v_procname, @v_currentlyActiveScopeModel, @p_TargetModel, @result, @v_stepname, @result, @result, 'success' ,@v_currentlyActiveScopeHash, @v_EntityIdHashKey);
		
	/**************************************		
		Step6: Set target model as current model
	***************************************/
	set @steperror = -6;
	set	@v_stepname='Step6: Set target model as current model';
	
	insert into scope_locker_history ( scope_id, scope_filter_hashcd ) values ( @p_TargetModel, @v_EntityIdHashKey );
		
	insert into @v_ErrorLog (scope_chglog_procnm, scope_chglog_target_from, scope_chglog_target_to, scope_chglog_step_cd, 
							 scope_chglog_step_nm, scope_chglog_proc_cd, scope_chglog_error_cd, scope_chglog_error_msg,
							 scope_chglog_target_from_hashcd, scope_chglog_target_to_hashcd)
	values (@v_procname, @v_currentlyActiveScopeModel, @p_TargetModel, @result, @v_stepname, @result, @result, 'success' ,@v_currentlyActiveScopeHash, @v_EntityIdHashKey);
	
	end try
	
	/***********************************************	
		CATCH error processing	
	 ***********************************************/
	
	begin catch
	
		select @v_errorcd = ERROR_NUMBER(),
			@v_errormsg = ERROR_MESSAGE(),
			@v_currentlyActiveScopeModel = coalesce(@v_currentlyActiveScopeModel,0)
			
		if @v_errorcd is not null 
			BEGIN
				if (@v_errorcd > 0)
					set @result = @v_errorcd*-1;
				else
					set @result = @v_errorcd;
			end
		else set @result = @steperror;	
		
		-- If Error is encountered in the procedure, rollback all changes
		IF @@TRANCOUNT > 0
		    ROLLBACK TRANSACTION;
		    
		
		insert into @v_ErrorLog (scope_chglog_procnm, scope_chglog_target_from, scope_chglog_target_to, scope_chglog_step_cd, 
								 scope_chglog_step_nm, scope_chglog_proc_cd, scope_chglog_error_cd, scope_chglog_error_msg,
								 scope_chglog_target_from_hashcd, scope_chglog_target_to_hashcd)
		values (@v_procname, @v_currentlyActiveScopeModel, @p_TargetModel, @result, @v_stepname, @result, @v_errorcd, @v_errormsg ,@v_currentlyActiveScopeHash, @v_EntityIdHashKey);
			
	end catch;
	
	-- If transaction is still active, no errors encountered, commit all changes
	
	IF @@TRANCOUNT > 0
    commit transaction;
	
	/*
		Insert all log rows to the logging table. Logging done like this to enable AUTONOMOUS "like" processing of log rows. Placing log rows into a
		table variable allows rows to persist when transaction is rolled back (ON ERROR).
	*/
	
	insert into dbo.scope_chglog_tbl 
		   (scope_chglog_id, scope_chglog_procnm, scope_chglog_target_from, scope_chglog_target_to, 
			scope_chglog_step_cd, scope_chglog_step_nm, scope_chglog_proc_cd, scope_chglog_error_cd, 
			scope_chglog_error_msg, scope_chglog_target_from_hashcd, scope_chglog_target_to_hashcd,
			scope_chglog_dtm, scope_chglog_row_cnt)
	select scope_chglog_id, scope_chglog_procnm, scope_chglog_target_from, scope_chglog_target_to, 
		   scope_chglog_step_cd, scope_chglog_step_nm, scope_chglog_proc_cd, scope_chglog_error_cd, 
		   scope_chglog_error_msg, scope_chglog_target_from_hashcd, scope_chglog_target_to_hashcd,
		   scope_chglog_dtm, scope_chglog_row_cnt
	from @v_ErrorLog
	order by scope_chglog_dtm asc;
	
	return @result;	
end
go

/* End: Escalation 25229:Duplicate Rows created in scope_info_history_locker and scope_loginfo_locker on each sync
================================================================================================*/
